package es

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_es.git/md"
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/olivere/elastic/v7"
)

func Ping(url string) (*elastic.PingResult, int, error) {
	return EsClient.Ping(url).Do(context.Background())
}

func ShowEsVersion(url string) string {
	esVersion, err := EsClient.ElasticsearchVersion(url)
	if err != nil {
		panic(err)
	}
	return esVersion
}

// CreateIndexIfNotExists 索引不存在时，创建索引
// index 索引名称
// mapping 数据类型
func CreateIndexIfNotExists(index, mapping string) error {
	ctx := context.Background()
	exists, err := EsClient.IndexExists(index).Do(ctx)
	if err != nil {
		return err
	}
	if exists {
		return nil
	}

	info, err := EsClient.CreateIndex(index).BodyString(mapping).Do(ctx)
	//info, err := EsClient.CreateIndex(index).Do(ctx) // 如果只是想创建索引时，那么就不需要 BodyString() 方法
	if err != nil {
		return err
	}
	if !info.Acknowledged {
		return errors.New(fmt.Sprintf("ES 创建索引 [%s] 失败", index))
	}
	return nil
}

// DeleteIndex 删除索引
// index 索引名称
func DeleteIndex(index string) (*elastic.IndicesDeleteResponse, error) {
	info, err := EsClient.DeleteIndex(index).Do(context.Background())
	if err != nil {
		return nil, err
	}
	if !info.Acknowledged {
		return nil, errors.New(fmt.Sprintf("ES 删除索引 [%s] 失败", index))
	}
	return info, err
}

// CreateDoc 单条添加
// index 索引
// id 文档 id（可以直接为空字符串，当实参为空字符串时，es 会主动随机生成）
// body 需要添加的内容
func CreateDoc(index, id string, body interface{}) (*elastic.IndexResponse, error) {
	client := EsClient.Index().Index(index)
	if "" != id {
		client = client.Id(id)
	}
	return client.BodyJson(body).Do(context.Background())
}

// UpdateDoc 单条更新
// index 索引
// id 记录 id
// body 需要更新的内容 （建议只使用 map[string]interface{} 进行更新指定字段且需要注意 map 中的 key 需要和 es 中的 key 完全匹配，否则 es 会认为新增字段，不要使用 struct 否则会将某些值初始化零值）
func UpdateDoc(index, id string, body interface{}) (*elastic.UpdateResponse, error) {
	return EsClient.Update().Index(index).Id(id).Doc(body).Do(context.Background())
}

// CreateBulkDoc 批量添加
// index 索引
// ids 需要新建的 id 数组（可以为空的字符串切片）
// body 需要添加的内容
// 需要注意：ids 和 body 的顺序要一一对应
func CreateBulkDoc(index string, ids []string, body []interface{}) (*elastic.BulkResponse, error) {
	bulkRequest := EsClient.Bulk()
	for k, v := range body {
		tmp := v
		doc := elastic.NewBulkIndexRequest().Index(index).Doc(tmp)
		if len(ids) > 0 {
			doc = doc.Id(ids[k])
		}
		bulkRequest = bulkRequest.Add(doc)
	}
	return bulkRequest.Do(context.Background())
}

// UpdateBulkDoc 批量更新
// index 索引
// ids 需要更新的 id 数组
// body 需要更新的 id 对应的数据 （建议只使用 []map[string]interface{} 进行更新指定字段且需要注意 map 中的 key 需要和 es 中的 key 完全匹配，否则 es 会认为新增字段，不要使用 struct 否则会将某些值初始化零值）
// 需要注意：ids 和 body 的顺序要一一对应
func UpdateBulkDoc(index string, ids []string, body []interface{}) (*elastic.BulkResponse, error) {
	bulkRequest := EsClient.Bulk()
	for k, v := range body {
		tmp := v
		doc := elastic.NewBulkUpdateRequest().Index(index).Id(ids[k]).Doc(tmp).DocAsUpsert(true)
		bulkRequest = bulkRequest.Add(doc)
	}
	return bulkRequest.Do(context.Background())
}

// DeleteBulkDoc 批量删除
// index 索引
// ids 需要删除的 id 数组
func DeleteBulkDoc(index string, ids []string) (*elastic.BulkResponse, error) {
	bulkRequest := EsClient.Bulk()
	for _, v := range ids {
		tmp := v
		req := elastic.NewBulkDeleteRequest().Index(index).Id(tmp)
		bulkRequest = bulkRequest.Add(req)
	}
	return bulkRequest.Do(context.Background())
}

// FirstDoc 通过 id 取出数据
// index 索引
// id 需要取的文档记录 id
func FirstDoc(index, id string) (*elastic.GetResult, error) {
	return EsClient.Get().Index(index).Id(id).Do(context.Background())
}

// DeleteDoc 删除文档
// index 索引
// id 需要删除的文档记录 id
func DeleteDoc(index, id string) (*elastic.DeleteResponse, error) {
	return EsClient.Delete().Index(index).Id(id).Do(context.Background())
}

// CountDoc 查询数据总数
func CountDoc(index string) int64 {
	// 查询总命中计数
	total, err := EsClient.Count().Index(index).Do(context.Background())
	if err != nil {
		panic(err)
	}
	return total
}

func PrintQuery(src interface{}) {
	fmt.Println("开始打印参数 ====>")
	data, err := json.MarshalIndent(src, "", "  ")
	if err != nil {
		panic(err)
	}
	fmt.Println(string(data))
	fmt.Println("打印参数结束 ====>")
}

func QuerySearch(query elastic.Query, index string) {
	if querySrc, err := query.Source(); err == nil {
		PrintQuery(querySrc)
	}
	queryRet, err := EsClient.Search().Index(index).Query(query).Do(context.Background())
	if err != nil {
		panic(err)
	}
	fmt.Printf("查询到的结果总数为 %v \n", queryRet.TotalHits())
	for _, v := range queryRet.Hits.Hits {
		var tmp md.ZhiosOrdersEs
		json.Unmarshal(v.Source, &tmp)
		fmt.Printf("已经命中查询的数据为 ==> %+v \n %+v \n\n", v.Id, tmp)
	}
}
